package repository

import (
	mgo "code.30cm.net/digimon/library-go/mongo"
	"context"
	"github.com/alicebob/miniredis/v2"
	"github.com/stretchr/testify/assert"
	"github.com/zeromicro/go-zero/core/stores/cache"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"testing"
	"time"
	"url_generate/internal/module/url/domain/entity"
	"url_generate/internal/module/url/domain/repository"
)

func SetupTestURLRepository(db string) (repository.URLRepository, func(), error) {
	h, p, tearDown, err := startMongoContainer()
	if err != nil {
		return nil, nil, err
	}
	s, _ := miniredis.Run()

	conf := &mgo.Conf{
		Schema:                           "mongodb",
		Host:                             h,
		Port:                             p,
		Database:                         db,
		MaxStaleness:                     300,
		MaxPoolSize:                      100,
		MinPoolSize:                      100,
		MaxConnIdleTime:                  300,
		Compressors:                      []string{},
		EnableStandardReadWriteSplitMode: false,
		ConnectTimeoutMs:                 3000,
	}

	cacheConf := cache.CacheConf{
		cache.NodeConf{
			RedisConf: redis.RedisConf{
				Host: s.Addr(),
				Type: redis.NodeType,
			},
			Weight: 100,
		},
	}

	cacheOpts := []cache.Option{
		cache.WithExpiry(1000 * time.Microsecond),
		cache.WithNotFoundExpiry(1000 * time.Microsecond),
	}

	param := URLRepositoryParam{
		conf:      conf,
		cacheConf: cacheConf,
		cacheOpts: cacheOpts,
	}

	repo := NewURLRepository(param)
	_, _ = repo.Index20241226001UP(context.Background())

	return repo, tearDown, nil
}

func TestAccountModel_Insert(t *testing.T) {
	repo, tearDown, err := SetupTestURLRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	tests := []struct {
		name        string
		account     *entity.URLTable
		expectError bool
	}{
		{
			name: "Valid account insert",
			account: &entity.URLTable{
				ShortCode: "123",
				URL:       "https://www.google.com",
			},
			expectError: false,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 插入測試帳戶
			err := repo.InsertMany(context.Background(), []*entity.URLTable{tt.account})
			if tt.expectError {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err, "插入操作應該成功")

				//// 檢查是否生成了 ObjectID 和時間戳
				//assert.NotZero(t, tt.account.ID, "ID 應該被生成")
				//assert.NotNil(t, tt.account.CreateAt, "CreateAt 應該被設置")
				//assert.NotNil(t, tt.account.UpdateAt, "UpdateAt 應該被設置")
				//
				//// 檢查插入的時間是否合理
				//now := time.Now().UTC().UnixNano()
				//assert.LessOrEqual(t, *tt.account.CreateAt, now, "CreateAt 應在當前時間之前")
				//assert.LessOrEqual(t, *tt.account.UpdateAt, now, "UpdateAt 應在當前時間之前")
				//
				//// 確認插入的資料
				//insertedAccount, err := repo.FindOne(context.Background(), tt.account.ID.Hex())
				//assert.NoError(t, err, "應該可以找到插入的帳號資料")
				//assert.Equal(t, tt.account.LoginID, insertedAccount.LoginID, "LoginID 應相同")
				//assert.Equal(t, tt.account.Token, insertedAccount.Token, "Token 應相同")
				//assert.Equal(t, tt.account.Platform, insertedAccount.Platform, "Platform 應相同")
			}
		})
	}
}
